# #!/usr/bin/python3
# -*- coding: utf-8 -*-
# @Time : 2023/5/29 9:31
# @Author : BruceLong
# @FileName: mail.py
# @Email   : 18656170559@163.com
# @Software: PyCharm
# @Blog ：http://www.cnblogs.com/yunlongaimeng/
import logging
import os
import smtplib
from email.header import Header
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.utils import formataddr


class EmailSender(object):
    def __init__(self, username, password, smtpserver="smtp.163.com", port=587, nickname='PYXBOX-邮件工具'):
        self.username = username
        self.password = password
        self.smtpserver = smtpserver
        self.smtp_client = smtplib.SMTP(smtpserver, port)
        self.nickname = nickname

    def __enter__(self):
        self.login()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.quit()

    def quit(self):
        self.smtp_client.quit()

    def login(self):  # 开启 TLS 加密
        self.smtp_client.starttls()
        self.smtp_client.login(self.username, self.password)

    def send(
            self,
            receivers: list,
            subject: str = '【测试】来自pyxbox工具',
            content: str = '测试邮件',
            cc_recipients: list = None,
            content_type: str = "plain",
            filepath: str = None,
    ):
        """

        Args:
            receivers:收件人列表
            subject:主题
            content:内容
            content_type: 内容的类型html / plain
            filepath:文件路径

        Returns:

        """
        # 创建一个带附件的实例
        message = MIMEMultipart()
        message["From"] = formataddr(
            (self.nickname, self.username)
        )  # 括号里的对应发件人邮箱昵称、发件人邮箱账号
        message["To"] = ",".join(
            [formataddr((receiver, receiver)) for receiver in receivers]
        )
        if cc_recipients:
            message["Cc"] = ",".join(cc_recipients)

        message["Subject"] = Header(subject, "utf-8")

        content = MIMEText(content, content_type, "utf-8")
        message.attach(content)

        # 构造附件
        if filepath:
            attach = MIMEText(open(filepath, "rb").read(), "base64", "utf-8")
            attach.add_header(
                "content-disposition",
                "attachment",
                filename=("utf-8", "", os.path.basename(filepath)),
            )
            message.attach(attach)

        msg = message.as_string()
        # 此处直接发送多个邮箱有问题，改成一个个发送
        for receiver in receivers:
            logging.debug("发送邮件到 {}".format(receiver))
            self.smtp_client.sendmail(self.username, receiver, msg)
        logging.debug("邮件发送成功！！！")
        return True





# #!/usr/bin/python3
# -*- coding: utf-8 -*-
# @Time : 2023/8/17 21:58
# @Author : BruceLong
# @FileName: weibo_utils.py
# @Email   : 18656170559@163.com
# @Software: PyCharm
# @Blog ：http://www.cnblogs.com/yunlongaimeng/
import re

str62keys = "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"


def get_id_from_url(url):
    mid = get_mid_from_url(url)
    if not mid:
        return ""
    return mid_to_id(mid)


def get_mid_from_url(url):
    if not url:
        return ""

    if '?' in url:
        url = url.split('?')[0]

    pattern = r'^https?://(?:[\w\-]+\.)?weibo\.com/[\w\-]+/(?P<id>[\w\-]+)$'
    match = re.match(pattern, url)
    if match:
        id = match.group("id")
        if id.isdigit():
            return id_to_mid(id)
        else:
            return id
    return ""


def mid_to_id(mid):
    id_ = ""
    for i in range(len(mid) - 4, -4, -4):
        offset1 = 0 if i < 0 else i
        length = len(mid) % 4 if i < 0 else 4
        substr = mid[offset1: offset1 + length]
        substr = str62_to_int(substr)
        if offset1 > 0:
            while len(substr) < 7:
                substr = "0" + substr
        id_ = substr + id_
    return id_


def id_to_mid(id_):
    mid = ""
    for i in range(len(id_) - 7, -7, -7):
        start_idx = 0 if i < 0 else i
        length = len(id_) % 7 if i < 0 else 7
        temp_str = id_[start_idx: start_idx + length]
        mid = int_to_str62(int(temp_str)) + mid
    return mid


def str62_to_int(str62):
    int10 = 0
    for i, char in enumerate(str62[::-1]):
        int10 += str62keys.index(char) * (62 ** i)
    return str(int10)


def int_to_str62(int10):
    s62 = ""
    while int10:
        r = int10 % 62
        s62 = str62keys[r] + s62
        int10 //= 62
    return s62



# #!/usr/bin/python3
# -*- coding: utf-8 -*-
# @Time : 2023/5/28 21:13
# @Author : BruceLong
# @FileName: media.py
# @Email   : 18656170559@163.com
# @Software: PyCharm
# @Blog ：http://www.cnblogs.com/yunlongaimeng/
import base64
import logging
import os
from typing import Optional, Dict

import ffmpeg
from tqdm import tqdm
from .tools import x_file
import requests


# *********************************************** 视频类工具 --start ***********************************************

class Video:
    def __init__(self):
        pass

    def download(self, url: str, save_path: str, proxies: Optional[Dict[str, None]] = None,
                 data: Optional[Dict[str, None]] = None, **kwargs):
        '''
        url:文件的 URL
        save_path:保存文件的路径
        proxies:{'http':"http://127.0.0.1:21882",'https':"http://127.0.0.1:21882"}
        data = {
            'key1': 'value1',
            'key2': 'value2'
        }
        '''
        directory = os.path.dirname(save_path)
        x_file.makedirs_file_path(directory)
        # 发送带有 stream=True 参数的 GET 请求,带有代理和数据的 GET 请求
        response = requests.get(url, stream=True, proxies=proxies, data=data, **kwargs)
        # 检查响应状态码是否为 200 OK
        if response.status_code == 200:
            total_size = int(response.headers.get('Content-Length', 0))
            block_size = 1024  # 每次下载的块大小
            progress_bar = tqdm(total=total_size, unit='B', unit_scale=True)

            with open(save_path, 'wb') as file:
                for data in response.iter_content(block_size):
                    progress_bar.update(len(data))
                    file.write(data)

            progress_bar.close()
            logging.info("File downloaded successfully.")
        else:
            logging.info("Failed to download file.")

    def convert_video(self, input_file, output_file, output_format):
        if any([x_file.is_exist(input_file), output_file, output_format]):
            logging.info('缺少参数')
        else:
            directory = os.path.dirname(output_file)
            x_file.makedirs_file_path(directory)
            try:
                ffmpeg.input(input_file).output(output_file, format=output_format).run()
                logging.info('格式转换完成')
            except ffmpeg.Error as e:
                logging.error(f"视频格式转换失败: {e.stderr.decode()}")


x_video = Video()


# *********************************************** 视频类工具 --end ***********************************************

# *********************************************** 图片类工具 --start ***********************************************

class Image:
    def __init__(self):
        pass

    def base64_to_jpg(self, b64str, save_path):
        data = base64.b64decode(b64str)
        directory = os.path.dirname(save_path)
        x_file.makedirs_file_path(directory)
        with open(''.format(save_path), 'wb') as f:
            f.write(data)
            return save_path

    def jpg_to_base64(self, jpg_path):
        if any([x_file.is_exist(jpg_path)]):
            logging.info('请输入正确的文件路径')
        else:
            with open(jpg_path, "rb") as image_file:
                encoded_string = base64.b64encode(image_file.read())
                return encoded_string.decode("utf-8")


x_image = Image()


# *********************************************** 图片类工具 --end ***********************************************


# *********************************************** 声音类工具 --end ***********************************************
class Sound:
    def __init__(self):
        import whisper
        self.model = whisper.load_model("base")

    def get_mp3_text(self, input_file):
        result = self.model.transcribe(input_file)
        return result

# *********************************************** 声音类工具 --end ***********************************************
